@@ -56,7 +56,13 @@ enum {
 	GRES_VAL_TYPE_ALLOC  = 3
 };
 
+typedef enum {
+	GRES_INTERNAL_FLAG_NONE = 0,
+	GRES_INTERNAL_FLAG_VERBOSE = 1 << 0,
+} gres_internal_flags_t;
+
 typedef struct {
+	int index;
 	int alloc;
 	int dev_num;
 	char *major;
@@ -72,9 +78,13 @@ typedef struct {
 #define GRES_NO_CONSUME		0x0001	/* Requesting no consume of resources */
 
 /* GRES AutoDetect options */
-#define GRES_AUTODETECT_NONE    0x00000000
-#define GRES_AUTODETECT_NVML    0x00000001
-#define GRES_AUTODETECT_RSMI    0x00000002
+#define GRES_AUTODETECT_UNSET     0x00000000 /* Not set */
+#define GRES_AUTODETECT_GPU_NVML  0x00000001
+#define GRES_AUTODETECT_GPU_RSMI  0x00000002
+#define GRES_AUTODETECT_GPU_OFF   0x00000004 /* Do NOT use global */
+
+#define GRES_AUTODETECT_GPU_FLAGS 0x000000ff /* reserve first 8 bits for gpu
+					      * flags */
 
 /* Gres state information gathered by slurmd daemon */
 typedef struct gres_slurmd_conf {
@@ -83,10 +93,12 @@ typedef struct gres_slurmd_conf {
 	/* Count of gres available in this configuration record */
 	uint64_t count;
 
-	/* Specific CPUs associated with this configuration record */
+	/* The # of CPUs on the node */
 	uint32_t cpu_cnt;
+	/* abstract/logical mapping range of cores */
 	char *cpus;
-	bitstr_t *cpus_bitmap;	/* Using LOCAL mapping */
+	/* machine/local/physical CPU mapping */
+	bitstr_t *cpus_bitmap;
 
 	/* Device file associated with this configuration record */
 	char *file;
@@ -184,6 +196,7 @@ typedef struct gres_job_state {
 	uint64_t gres_per_socket;
 	uint64_t gres_per_task;
 	uint64_t mem_per_gres;
+	uint16_t ntasks_per_gres;
 
 	/*
 	 * Default GRES configuration parameters. These values are subject to
@@ -269,7 +282,7 @@ typedef struct gres_step_state {
 
 /* Per-socket GRES availability information for scheduling purposes */
 typedef struct sock_gres {	/* GRES availability by socket */
-	bitstr_t *bits_any_sock;/* Per-socket GRES bitmap of this name & type */
+	bitstr_t *bits_any_sock;/* GRES bitmap unconstrained by cores */
 	bitstr_t **bits_by_sock;/* Per-socket GRES bitmap of this name & type */
 	uint64_t cnt_any_sock;	/* GRES count unconstrained by cores */
 	uint64_t *cnt_by_sock;	/* Per-socket GRES count of this name & type */
@@ -299,6 +312,7 @@ typedef struct gres_mc_data {
 	uint16_t ntasks_per_board;  /* number of tasks to invoke on each board */
 	uint16_t ntasks_per_socket; /* number of tasks to invoke on each socket */
 	uint16_t ntasks_per_core;   /* number of tasks to invoke on each core */
+	uint16_t ntasks_per_tres; /* number of tasks that can access each gpu */
 	uint8_t overcommit;         /* processors being over subscribed */
 	uint16_t plane_size;        /* plane size for SLURM_DIST_PLANE */
 	uint32_t task_dist;         /* task distribution directives */
@@ -393,10 +407,10 @@ extern int gres_plugin_node_config_pack(Buf buffer);
 extern List gres_plugin_get_allocated_devices(List gres_list, bool is_job);
 
 /* Send GRES information to slurmstepd on the specified file descriptor */
-extern void gres_plugin_send_stepd(int fd);
+extern void gres_plugin_send_stepd(int fd, slurm_msg_t *msg);
 
 /* Receive GRES information from slurmd on the specified file descriptor */
-extern void gres_plugin_recv_stepd(int fd);
+extern void gres_plugin_recv_stepd(int fd, slurm_msg_t *msg);
 
 /*
  **************************************************************************
@@ -569,27 +583,6 @@ extern int gres_plugin_node_count(List gres_list, int arr_len,
 				  uint64_t *gres_count_vals,
 				  int val_type);
 
-/*
- * Fill in an array of GRES type ids contained within the given job gres_list
- *		and an array of corresponding counts of those GRES types.
- * IN gres_list - a List of GRES types allocated to a job.
- * IN arr_len - Length of the arrays (the number of elements in the gres_list).
- * IN gres_count_ids, gres_count_vals - the GRES type ID's and values found
- *	 	in the gres_list.
- * RET SLURM_SUCCESS or error code
- */
-extern int gres_plugin_job_count(List gres_list, int arr_len,
-				 uint32_t *gres_count_ids,
-				 uint64_t *gres_count_vals);
-
-/*
- * Build a string identifying total GRES counts of each type
- * IN gres_list - a List of GRES types allocated to a job.
- * RET string containing comma-separated list of gres type:model:count
- *     must release memory using xfree()
- */
-extern char *gres_plugin_job_alloc_count(List gres_list);
-
 /*
  * Pack a job's allocated gres information for use by prolog/epilog
  * IN gres_list - generated by gres_plugin_job_config_validate()
@@ -648,6 +641,7 @@ extern void gres_plugin_epilog_set_env(char ***epilog_env_ptr,
  *		      provide consistent gres_per_socket/node values
  * IN/OUT cpus_per_task - requested ntasks_per_socket count, may be reset to
  *		      provide consistent gres_per_task/cpus_per_gres values
+ * IN/OUT ntasks_per_tres - requested ntasks_per_tres count
  * OUT gres_list - List of GRES records for this job to track usage
  * RET SLURM_SUCCESS or ESLURM_INVALID_GRES
  */
@@ -665,6 +659,7 @@ extern int gres_plugin_job_state_validate(char *cpus_per_tres,
 					  uint16_t *ntasks_per_socket,
 					  uint16_t *sockets_per_node,
 					  uint16_t *cpus_per_task,
+					  uint16_t *ntasks_per_tres,
 					  List *gres_list);
 
 /*
@@ -875,6 +870,7 @@ extern List gres_plugin_job_test2(List job_gres_list, List node_gres_list,
  * IN cpus_per_core   - Count of CPUs per core on this node
  * IN sock_per_node   - sockets requested by job per node or NO_VAL
  * IN task_per_node   - tasks requested by job per node or NO_VAL16
+ * IN cpus_per_task   - Count of CPUs per task
  * IN whole_node      - we are requesting the whole node or not
  * OUT avail_gpus     - Count of available GPUs on this node
  * OUT near_gpus      - Count of GPUs available on sockets with available CPUs
@@ -889,6 +885,7 @@ extern int gres_plugin_job_core_filter2(List sock_gres_list, uint64_t avail_mem,
 					uint16_t cpus_per_core,
 					uint32_t sock_per_node,
 					uint16_t task_per_node,
+					uint16_t cpus_per_task,
 					bool whole_node,
 					uint16_t *avail_gpus,
 					uint16_t *near_gpus);
@@ -980,6 +977,23 @@ extern int gres_plugin_job_min_cpus(uint32_t node_count,
 				    uint32_t task_count,
 				    List job_gres_list);
 
+/*
+ * Determine the minimum number of tasks required to satisfy the job's GRES
+ *	request (based upon total GRES times ntasks_per_tres value). If
+ *	ntasks_per_tres is not specified, returns 0.
+ * node_count IN - count of nodes in job allocation
+ * sockets_per_node IN - count of sockets per node in job allocation
+ * ntasks_per_tres IN - # of tasks per GPU
+ * gres_name IN - (optional) Filter GRES by name. If NULL, check all GRES
+ * job_gres_list IN - job GRES specification
+ * RET count of required tasks for the job
+ */
+extern int gres_plugin_job_min_tasks(uint32_t node_count,
+				     uint32_t sockets_per_node,
+				     uint16_t ntasks_per_tres,
+				     char *gres_name,
+				     List job_gres_list);
+
 /*
  * Determine the minimum number of CPUs required to satify the job's GRES
  *	request on one node
@@ -1104,10 +1118,13 @@ extern void gres_plugin_job_set_env(char ***job_env_ptr, List job_gres_list,
  *		  gres_name="gpu" would apply to "gpu:tesla", "gpu:volta", etc.)
  * IN cpu_per_gpu - value to set as default
  * IN mem_per_gpu - value to set as default
+ * OUT *cpus_per_tres - CpusPerTres string displayed by scontrol show job
+ * OUT *mem_per_tres - MemPerTres string displayed by scontrol show job
  */
 extern void gres_plugin_job_set_defs(List job_gres_list, char *gres_name,
-				     uint64_t cpu_per_gpu,
-				     uint64_t mem_per_gpu);
+				     uint64_t cpu_per_gpu, uint64_t mem_per_gpu,
+				     char **cpus_per_tres, char **mem_per_tres,
+				     uint16_t *cpus_per_task);
 
 /*
  * Extract from the job record's gres_list the count of allocated resources of
@@ -1141,9 +1158,12 @@ extern int gres_plugin_step_state_validate(char *cpus_per_tres,
 					   char *tres_per_socket,
 					   char *tres_per_task,
 					   char *mem_per_tres,
+					   uint16_t ntasks_per_tres,
 					   List *step_gres_list,
 					   List job_gres_list, uint32_t job_id,
-					   uint32_t step_id);
+					   uint32_t step_id,
+					   uint32_t *num_tasks,
+					   uint32_t *cpu_count);
 
 /*
  * Create a copy of a step's gres state
@@ -1175,20 +1195,20 @@ void gres_plugin_step_state_rebase(List gres_list,
  * Pack a step's current gres status, called from slurmctld for save/restore
  * IN gres_list - generated by gres_plugin_step_allocate()
  * IN/OUT buffer - location to write state to
- * IN job_id, step_id - job and step ID for logging
+ * IN step_id - job and step ID for logging
  */
 extern int gres_plugin_step_state_pack(List gres_list, Buf buffer,
-				       uint32_t job_id, uint32_t step_id,
+				       slurm_step_id_t *step_id,
 				       uint16_t protocol_version);
 
 /*
  * Unpack a step's current gres status, called from slurmctld for save/restore
  * OUT gres_list - restored state stored by gres_plugin_step_state_pack()
  * IN/OUT buffer - location to read state from
- * IN job_id, step_id - job and step ID for logging
+ * IN step_id - job and step ID for logging
  */
 extern int gres_plugin_step_state_unpack(List *gres_list, Buf buffer,
-					 uint32_t job_id, uint32_t step_id,
+					 slurm_step_id_t *step_id,
 					 uint16_t protocol_version);
 
 /* Return the count of GRES of a specific name on this machine
@@ -1363,7 +1383,7 @@ extern int gres_get_step_info(List step_gres_list, char *gres_name,
 extern gres_job_state_t *gres_get_job_state(List gres_list, char *name);
 extern gres_step_state_t *gres_get_step_state(List gres_list, char *name);
 
-extern uint32_t gres_get_autodetect_types(void);
+extern uint32_t gres_get_autodetect_flags(void);
 
 /*
  * Translate a gres_list into a tres_str
@@ -1375,6 +1395,18 @@ extern uint32_t gres_get_autodetect_types(void);
  */
 extern char *gres_2_tres_str(List gres_list, bool is_job, bool locked);
 
+/* Given a job's GRES data structure, return a simple tres string of gres
+ * allocated on the node_inx requested
+ * IN job_gres_list  - job's GRES data structure
+ * IN node_inx - position of node in job_state_ptr->gres_cnt_node_alloc
+ *
+ * RET - simple string containing gres this job is allocated on the node
+ * requested.
+ */
+extern char *gres_job_gres_on_node_as_tres(List job_gres_list,
+					   int node_inx,
+					   bool locked);
+
 /* Fill in the job allocated tres_cnt based off the gres_list and node_cnt
  * IN gres_list - filled in with gres_job_state_t's
  * IN node_cnt - number of nodes in the job
